@@ -155,8 +155,21 @@ Field types
 .. autoclass:: Pickle
    
 .. autoclass:: Compressed
+
+Utility functions
+`````````````````
+
+.. autofunc:: raw_unpack
+
+.. autofunc:: header
+
+.. autofunc:: version
+
+.. autofunc:: full_header
 '''
-__all__ = ['BinPack', 'Field', 'Bytes', 'Pickle', 'Compressed']
+__all__ = ['BinPack', 'Field', 'Bytes', 'Pickle', 'Compressed',
+           'raw_unpack', 'header', 'version', 'full_header']
+
 import collections
 try:
     from cStringIO import StringIO
@@ -174,6 +187,7 @@ class voiddict(collections.MutableMapping):
     def __len__(self):
         return 0
 
+BinPack = None
 class BinPack(collections.Mapping):
     '''Packs binary data in a file.
     
@@ -198,10 +212,36 @@ class BinPack(collections.Mapping):
         (read or write) only once.
     '''
 
-    __dict = {}
+    class __metaclass__(type(collections.Mapping)):
+        def __new__(cls, name, bases, namespace):
+
+            # Initializes the BinPack class
+            if BinPack is None:
+                return type(collections.Mapping).__new__(cls, name, bases, namespace)
+
+            # Initializes a BinPack sub-class
+            else:
+                # Check bases
+                if len(bases) > 1:
+                    raise TypeError('BinPack subclasses do not accept multiple inheritance')
+
+                # Ensure the locking mechanism
+                base = bases[0]
+                if not base.is_locked():
+                    raise TypeError('Cannot subclass unlocked classes')
+
+                # Creates new type and update its class dictionary
+                new = type(collections.Mapping).__new__(cls, name, bases, namespace)
+                if base is not BinPack:
+                    new._cls_dict_ = {'fields': base._cls_dict_['fields'].copy()}
+                else:
+                    new._cls_dict_ = {}
+                return new
+
+    _cls_dict_ = {}
 
     def __new__(cls, *args, **kwds):
-        if cls.is_ready():
+        if cls.is_locked():
             return object.__new__(cls)
         else:
             tname = cls.__name__
@@ -239,7 +279,7 @@ class BinPack(collections.Mapping):
 
         # Reads/writes the header/version to file
         if self.mode == 'r':
-            self._data = self._data_read()
+            self._data = raw_unpack(self.file)
             header = next(self._data)
             version = next(self._data)
 
@@ -248,6 +288,8 @@ class BinPack(collections.Mapping):
             if version != self.get_version():
                 raise ValueError("bad version: %s" % repr(version))
 
+            self._fields_it = iter(self._fields)
+            self._data = ((next(self._fields_it), data) for data in self._data)
         else:
             self.file.write(self._cls_dict['full_header'])
 
@@ -293,23 +335,15 @@ class BinPack(collections.Mapping):
 
     def _write(self, data):
         self._assure_opened()
-        self.file.write(self._bin_num(len(data)))
+        self.file.write(bytes7num(len(data)))
         self.file.write(data)
 
-    def _bin_num(self, n):
-        data = []
-        while n:
-            n, mod = divmod(n, 128)
-            data.append(chr(mod))
-        data = ''.join(data)
-        return data
-
     #===========================================================================
     # Properties
     #===========================================================================
     @property
     def _cls_dict(self):
-        return type(self).__dict
+        return type(self)._cls_dict_
 
     #===========================================================================
     # Abstract methods
@@ -343,7 +377,7 @@ class BinPack(collections.Mapping):
                 if k == key:
                     return self[key]
 
-            raise RuntimeError('should never reach here!')
+            raise RuntimeError('should never reach here\n\tkey: %s\n\tdata: %s' % (key, self._raw.keys()))
 
         # In write mode, it checks for default values
         else:
@@ -386,63 +420,6 @@ class BinPack(collections.Mapping):
                 self._ok.append(k)
                 self._processed.add(k)
 
-    #===========================================================================
-    # Reading data
-    #===========================================================================
-    def _read_num(self, F):
-        nums = []
-        n = 0
-        while True:
-            try:
-                n = ord(F.read(1))
-            except TypeError: # F.read(1) is empty
-                return None
-
-            if n > 127:
-                nums.append(n - 128)
-            else:
-                nums.append(n)
-                break
-
-        return sum(n * (1 << (i * 7)) for (i, n) in enumerate(nums))
-
-    def _data_read(self):
-        '''Generator that reads data from file'''
-
-        F = self.file
-        read_num = self._read_num
-        sep = self._cls_dict['sep']
-        is_binary = self._cls_dict['binary']
-
-        if is_binary:
-            F.read(1)
-
-        # Yield header
-        c, chars = 1, []
-        while c != sep and c:
-            c = F.read(1)
-            chars.append(c)
-        chars.pop()
-        yield ''.join(chars)
-
-        # Yield version
-        c, chars = 1, []
-        while c != sep and c:
-            c = F.read(1)
-            chars.append(c)
-        chars.pop()
-        yield ''.join(chars)
-
-        # Yield fields
-        for key in self:
-            n = read_num(F)
-            if n is None:
-                raise ValueError('data stream ended before expected')
-            yield (key, F.read(n))
-
-        remaining = F.read()
-        assert not remaining, 'some data was not read: %s' % remaining
-
     #===========================================================================
     # Subtypes creation
     #===========================================================================
@@ -450,14 +427,15 @@ class BinPack(collections.Mapping):
     def new_type(cls, name, header=None, version=None, binary=True):
         '''Returns a new BinPack sub-type'''
 
-        cls._assure_locked()
-        new = type(name, (cls,), {})
-        new.__dict = {}
+        class NewType(cls):
+            pass
+        NewType.__name__ = name
+
         if header is not None:
-            new.set_header(header)
+            NewType.set_header(header)
         if version is not None:
-            new.set_version(version)
-        return new
+            NewType.set_version(version)
+        return NewType
 
 #TODO: implement versioning
 #    @classmethod
@@ -469,16 +447,16 @@ class BinPack(collections.Mapping):
     # Locking mechanism
     #===========================================================================
     @classmethod
-    def is_ready(cls):
+    def is_locked(cls):
         '''Return True when class is ready to instantiate methods'''
 
-        return cls.__dict.get('locked', False)
+        return cls._cls_dict_.get('locked', False)
 
     @classmethod
     def lock(cls):
         '''Prevents class from changing and allows creation of instances'''
 
-        dic = cls.__dict
+        dic = cls._cls_dict_
 
         # Check if class has the header and version are set properly
         if 'header' not in dic:
@@ -505,12 +483,12 @@ class BinPack(collections.Mapping):
 
     @classmethod
     def _assure_unlocked(cls):
-        if cls.__dict.get('locked', False):
+        if cls._cls_dict_.get('locked', False):
             raise RuntimeError('class must be unlocked in order to accept changes')
 
     @classmethod
     def _assure_locked(cls):
-        if not cls.__dict.get('locked', False):
+        if not cls._cls_dict_.get('locked', False):
             raise RuntimeError('class must be locked in order to perform this operation')
 
     #===========================================================================
@@ -523,8 +501,8 @@ class BinPack(collections.Mapping):
 
         cls._assure_unlocked()
         header = unicode(header, 'ascii').encode('ascii')
-        cls.__dict['header'] = header
-        cls.__dict['binary'] = True
+        cls._cls_dict_['header'] = header
+        cls._cls_dict_['binary'] = True
 
     @classmethod
     def set_version(cls, version):
@@ -534,14 +512,14 @@ class BinPack(collections.Mapping):
 
         cls._assure_unlocked()
         version = unicode(version, 'ascii').encode('ascii')
-        cls.__dict['version'] = version
+        cls._cls_dict_['version'] = version
 
     @classmethod
     def add_field(cls, field, converter=None, force=False):
         '''Register a field associating a data type and a default value.'''
 
         cls._assure_unlocked()
-        fields = cls.__dict.setdefault('fields', collections.OrderedDict())
+        fields = cls._cls_dict_.setdefault('fields', collections.OrderedDict())
         if field in fields and not force:
             raise ValueError("field '%s' already exists!" % field)
 
@@ -561,7 +539,7 @@ class BinPack(collections.Mapping):
         '''Register a field associating a data type and a default value.'''
 
         cls._assure_unlocked()
-        fields = cls.__dict.setdefault('fields', collections.OrderedDict())
+        fields = cls._cls_dict_.setdefault('fields', collections.OrderedDict())
         del fields[field]
 
     @classmethod
@@ -569,7 +547,7 @@ class BinPack(collections.Mapping):
         '''Register a field before the given key or index.'''
 
         cls._assure_unlocked()
-        fields = cls.__dict.setdefault('fields', collections.OrderedDict())
+        fields = cls._cls_dict_.setdefault('fields', collections.OrderedDict())
         if field in fields:
             raise ValueError("field '%s' already exists" % field)
 
@@ -578,7 +556,7 @@ class BinPack(collections.Mapping):
         head, tail = items[:idx], items[idx:]
 
         fields = collections.OrderedDict(head)
-        cls.__dict['fields'] = fields
+        cls._cls_dict_['fields'] = fields
         try:
             cls.add_field(field, converter)
         finally:
@@ -589,11 +567,15 @@ class BinPack(collections.Mapping):
     #===========================================================================
     @classmethod
     def get_header(cls):
-        return cls.__dict['header']
+        return cls._cls_dict_['header']
 
     @classmethod
     def get_version(cls):
-        return cls.__dict['version']
+        return cls._cls_dict_['version']
+
+    @classmethod
+    def get_fields(cls):
+        return cls._cls_dict_['fields'].copy()
 
 #===============================================================================
 # Converter fields
@@ -643,50 +625,107 @@ class Bytes(Field):
     def decoder(self, data):
         return data
 
-class Pickle(Field):
-    '''Serialize python objects using the pickle protocol'''
+class Serializer(Field):
+    def serializers(self):
+        '''It must return a tuple of (encoder, decoder) functions. The arguments
+        from the init method are passed to this function by default. 
+        
+        e.g., the Pickle serializer return the pickle.dumps, and pickle.loads 
+        functions.
+        '''
+        raise NotImplementedError('must be implemented in child classes')
 
-    @classmethod
-    def pickle(cls):
+    def __init__(self, default=None, *args, **kwds):
+        super(Serializer, self).__init__(default)
+        self._encoder, self._decoder = self.serializers(*args, **kwds)
+
+    def encoder(self, obj):
+        try:
+            return self._encoder(obj)
+        except:
+            print obj
+            raise
+
+    def decoder(self, data):
+        try:
+            return self._decoder(data)
+        except:
+            print repr(data)
+            raise
+
+class Pickle(Serializer):
+    '''Serialize python objects using the pickle protocol
+    
+    Examples
+    --------
+    
+    >>> pickle_file = BinPack.new_type('json_file', 'JSON', '0.1')
+    >>> pickle_file.add_field('data', Pickle())
+    >>> pickle_file.lock()
+    >>> F = StringIO()
+    >>> with pickle_file(F, 'w') as data:
+    ...     data['data'] = [None, 1, 'two']
+    >>> F.seek(0)
+    >>> with pickle_file(F) as data:
+    ...     print(data['data'])
+    [None, 1, 'two']
+    '''
+
+    def serializers(self, protocol):
         '''Loads pickle module'''
 
         try:
-            return cls._pickle
-        except AttributeError:
-            try:
-                import cPickle as pickle
-            except ImportError:
-                import pickle
+            import cPickle as pickle
+        except ImportError:
+            import pickle
+        import functools
 
-            cls._pickle = pickle
-            return pickle
+        dumps = functools.partial(pickle.dumps, protocol=protocol)
+        return (dumps, pickle.loads)
 
-    def encoder(self, obj):
-        pickle = self.pickle()
-        return pickle.dumps(obj, 2)
+    def __init__(self, default=None, protocol=2):
+        super(Pickle, self).__init__(default, protocol)
 
-    def decoder(self, data):
-        pickle = self.pickle()
-        return pickle.loads(data)
+class JSON(Serializer):
+    '''Serialize a JSON structure.
+    
+    Examples
+    --------
+    
+    >>> json_file = BinPack.new_type('json_file', 'JSON', '0.1')
+    >>> json_file.add_field('data', JSON())
+    >>> json_file.lock()
+    >>> F = StringIO()
+    >>> with json_file(F, 'w') as data:
+    ...     data['data'] = {'foo': 'bar', 'spam': ['ham', 'eggs']}
+    >>> F.getvalue()                                        # doctest: +ELLIPSIS
+    '...{"foo": "bar", "spam": ["ham", "eggs"]}'
+    '''
+
+    def serializers(self):
+        '''Loads dumps and loads functions from the simplejson or json modules'''
 
-class Compressed(Field):
-    def __init__(self, filter, method='zlib'): #@ReservedAssignment
-        '''Compressed can be applied to any field in order to compress an 
-        arbitrary data stream. 
-        
-        Parameters
-        ----------
-        filter : Filter instance
-            The original filter. Its data will be compressed in the file.
-        method : str
-            A string describing the compression method. Currently, only 'bz2' 
-            and 'zlib' are supported
-        '''
         try:
-            compress_funcs = getattr(self, 'compress_module_' + method)
-        except AttributeError:
-            raise ValueError("compression method '%s' is not supported" % method)
-        self.compress, self.decompress = compress_funcs()
+            import simplejson as json
+        except ImportError:
+            import json
+
+        return (json.dumps, json.loads)
+
+class Compressed(Serializer):
+    '''Compressed can be applied to any field in order to compress an 
+    arbitrary data stream. 
+    
+    Parameters
+    ----------
+    filter : Filter instance
+        The original filter. Its data will be compressed in the file.
+    method : str
+        A string describing the compression method. Currently, only 'bz2' 
+        and 'zlib' are supported
+    '''
+    def __init__(self, filter, method='zlib', **kwds): #@ReservedAssignment
+        self.compress, self.decompress = self.method(method, **kwds)
         self.filter = filter
 
     def encoder(self, obj):
@@ -708,14 +747,15 @@ class Compressed(Field):
     # Compressing methods
     #===========================================================================
     @classmethod
-    def compress_module_bz2(cls):
-        import bz2
-        return bz2.compress, bz2.decompress
-
-    @classmethod
-    def compress_module_zlib(cls):
-        import zlib
-        return zlib.compress, zlib.decompress
+    def method(cls, method, **kwds):
+        if method == 'bz2':
+            import bz2
+            return bz2.compress, bz2.decompress
+        elif method == 'zlib':
+            import zlib
+            return zlib.compress, zlib.decompress
+        else:
+            raise ValueError("compression method '%s' is not supported" % method)
 
 #===============================================================================
 # Locks BinPack class
@@ -725,6 +765,118 @@ BinPack.set_version('0.1')
 BinPack.add_field('data')
 BinPack.lock()
 
+#===============================================================================
+# Utility functions
+#===============================================================================
+def bytes7num(n):
+    '''Return number as a string of terms that can be read by read_num()'''
+    data = []
+    if n == 0:
+        return chr(0)
+    else:
+        while n:
+            n, mod = divmod(n, 128)
+            data.append(mod)
+        for idx in range(len(data) - 1):
+            data[idx] = data[idx] + 128
+        return ''.join(map(chr, data))
+
+def read_num(F):
+    '''Read the necessary number of bytes < 128 from F and construct a base-2^7
+    number as output.'''
+    nums = []
+    n = 0
+    while True:
+        try:
+            n = ord(F.read(1))
+        except TypeError: # F.read(1) is empty
+            if nums:
+                raise ValueError('File ended before expected (%s bytes read)' % len(nums))
+            else:
+                return None
+
+        if n > 127:
+            nums.append(n - 128)
+        else:
+            nums.append(n)
+            break
+    return sum(n * (1 << (i * 7)) for (i, n) in enumerate(nums))
+
+def raw_unpack(F, sep='\xff'):
+    '''Iterates over all fields and returns raw data.
+    
+    The iterator that results from raw_upack(F) returns the header, version and
+    each raw data field in the file object.  
+    '''
+
+    if sep == '\xff':
+        byte = F.read(1)
+        if byte != '\xff':
+            raise ValueError('unexpected first byte: %s' % ord(byte))
+
+    # Yield header
+    c, chars = 1, []
+    while c != sep and c:
+        c = F.read(1)
+        chars.append(c)
+    chars.pop()
+    yield ''.join(chars)
+
+    # Yield version
+    c, chars = 1, []
+    while c != sep and c:
+        c = F.read(1)
+        chars.append(c)
+    chars.pop()
+    yield ''.join(chars)
+
+    # Yield fields
+    n = 0
+    while n is not None:
+        n = read_num(F)
+        if n is not None:
+            yield F.read(n)
+
+    remaining = F.read()
+    assert not remaining, 'some data was not read: %s' % remaining
+
+def header(F):
+    '''Returns the header of a file.'''
+
+    try:
+        pos = F.tell()
+        header = next(raw_unpack(F))
+    finally:
+        F.seek(pos)
+    return header
+
+def version(F):
+    '''Returns the version of a file.'''
+
+    return full_header(F)[1]
+
+def full_header(F):
+    '''Returns a tuple with (header, version) for the file F.'''
+
+    try:
+        pos = F.tell()
+        header = next(raw_unpack(F))
+        version = next(raw_unpack(F))
+    finally:
+        F.seek(pos)
+    return (header, version)
+
 if __name__ == '__main__':
+#    from StringIO import StringIO
+#    F = StringIO()
+#    with BinPack(F, mode='w') as data:
+#        data['data'] = 'fdsdfsd sds'
+#    print F.getvalue()
+#
+#    F2 = StringIO(F.getvalue())
+#    print header(F2)
+#    with BinPack(F2) as data:
+#        print data['data']
+
     import doctest
-    doctest.testmod(optionflags=doctest.REPORT_ONLY_FIRST_FAILURE, verbose=0)
+#    doctest.testmod(optionflags=doctest.REPORT_ONLY_FIRST_FAILURE, verbose=0)
